import JGTConfig as jgtc
import dropbox
import pathlib
from dropbox.exceptions import AuthError
import pandas as pd
#@title Functions DROPBOX Api
import jgtfxcon.JGTCore as jko
import os
from pathlib import Path
import pathlib


_DROPBOX_ACCESS_TOKEN=jgtc._DROPBOX_ACCESS_TOKEN
def mkdir_existok(tpath):
    pathlib.Path(tpath).mkdir(parents=True, exist_ok=True)
     
def etc_dl_json_as_dict(fn,subdir='.'):
    tmpdir = 'tmp'
    etc_dl(fn,tmpdir,subdir)
    tfile = os.path.join(tmpdir,fn)
    print(tfile)
    dictio= jko.jsonfile2prop(tfile)
    os.remove(tfile)
    return dictio
               
def etc_dl(fn,local_tdir='.',etcsubdir='.'):
    mkdir_existok(local_tdir)
    local_file_path =  local_tdir + '/'+ fn
    #os.path.join(local_tdir,fn)
    print('Local file: ' + local_file_path)
    etcpath = jgtc.DROPBOX_ETC_PATH
    if etcsubdir != '.':
        etcpath = etcpath + '/' + etcsubdir
    etcpath=etcpath.replace('//','/')
    print('etc path:' + etcpath)
    fnpath= etcpath +'/'+ fn
    fnpath= fnpath.replace('//','/')
    print('fnpath: ' + fnpath)
    meta=dropbox_download_file(fnpath,local_file_path)
    print(meta)




def dropbox_connect():
    """Create a connection to Dropbox."""

    try:
        dbx = dropbox.Dropbox(_DROPBOX_ACCESS_TOKEN)
    except AuthError as e:
        print('Error connecting to Dropbox with access token: ' + str(e))
    return dbx
    

def dxls(path='/'):
	"""Return List dropbox files as dataframe
	alias of dropbox_list_file
	"""
	return dropbox_list_files(path)


def dropbox_list_files(path):
    """Return a Pandas dataframe of files in a given Dropbox folder path in the Apps directory.
    """

    dbx = dropbox_connect()

    try:
        files = dbx.files_list_folder(path).entries
        files_list = []
        for file in files:
            if isinstance(file, dropbox.files.FileMetadata):
                metadata = {
                    'name': file.name,
                    'path_display': file.path_display,
                    'client_modified': file.client_modified,
                    'server_modified': file.server_modified
                }
                files_list.append(metadata)

        df = pd.DataFrame.from_records(files_list)
        return df.sort_values(by='server_modified', ascending=False)

    except Exception as e:
        print('Error getting list of files from Dropbox: ' + str(e))


def np_dropbox_download_csv(dropbox_file_path):
    """Download a file from Dropbox to the local machine."""

    try:
        dbx = dropbox_connect()

        
        metadata, result = dbx.files_download(path=dropbox_file_path)
        #print(metadata)
        return result.content
        #f.write(result.content)
    except Exception as e:
        print('Error downloading file from Dropbox: ' + str(e))
        
def jgt_dropbox_download_dic(dropbox_file_path):
    """Download a file from Dropbox to the local machine."""

    try:
        dbx = dropbox_connect()
        
        metadata, result = dbx.files_download(path=dropbox_file_path)
        #print(metadata)
        return jko.cnf_Decoder(result.content)
        #f.write(result.content)
    except Exception as e:
        print('Error downloading file from Dropbox: ' + str(e))
        
def dropbox_download_file(dropbox_file_path, local_file_path):
    """Download a file from Dropbox to the local machine."""

    try:
        dbx = dropbox_connect()

        with open(local_file_path, 'wb') as f:
            metadata, result = dbx.files_download(path=dropbox_file_path)
            f.write(result.content)
    except Exception as e:
        print('Error downloading file from Dropbox: ' + str(e))
        

def dxu(local_file,dxfilepath):
	local_path ='TODO'
	return dropbox_upload_file(local_path,local_file,dxfilepath)


def dropbox_upload_file(local_path, local_file, dropbox_file_path):
    """Upload a file from the local machine to a path in the Dropbox app directory.

    Args:
        local_path (str): The path to the local file.
        local_file (str): The name of the local file.
        dropbox_file_path (str): The path to the file in the Dropbox app directory.

    Example:
        dropbox_upload_file('.', 'test.csv', '/stuff/test.csv')

    Returns:
        meta: The Dropbox file metadata.
    """

    try:
        dbx = dropbox_connect()

        local_file_path = pathlib.Path(local_path) / local_file

        with local_file_path.open("rb") as f:
            meta = dbx.files_upload(f.read(), dropbox_file_path, mode=dropbox.files.WriteMode("overwrite"))

            return meta
    except Exception as e:
        print('Error uploading file to Dropbox: ' + str(e))
        
        
        
        
def dropbox_get_link(dropbox_file_path):
    """Get a shared link for a Dropbox file path.

    Args:
        dropbox_file_path (str): The path to the file in the Dropbox app directory.

    Returns:
        link: The shared link.
    """

    try:
        dbx = dropbox_connect()
        shared_link_metadata = dbx.sharing_create_shared_link_with_settings(dropbox_file_path)
        shared_link = shared_link_metadata.url
        return shared_link.replace('?dl=0', '?dl=1')
    except dropbox.exceptions.ApiError as exception:
        if exception.error.is_shared_link_already_exists():
            shared_link_metadata = dbx.sharing_get_shared_links(dropbox_file_path)
            shared_link = shared_link_metadata.links[0].url
            return shared_link.replace('?dl=0', '?dl=1')





# Copyright 2023 Jean Guillaume Isabelle
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
.. moduleauthor:: Jean Guillaume Isabelle <jgi@jgwill.com>
"""

# from jgtfxcommon.BatchOrderMonitor import BatchOrderMonitor
# from jgtfxcommon.OrderMonitor import OrderMonitor
# from jgtfxcommon.OrderMonitorNetting import OrderMonitorNetting
# from jgtfxcommon.TableListenerContainer import TableListenerContainer
# from jgtfxcommon.common import add_main_arguments, add_instrument_timeframe_arguments, \
#     add_candle_open_price_mode_argument, add_direction_rate_lots_arguments, add_account_arguments, \
#     valid_datetime, add_date_arguments, add_report_date_arguments, add_max_bars_arguments, add_bars_arguments, \
#     print_exception, session_status_changed, diff_month, convert_timeframe_to_seconds

import os
import platform
import sys
from . import jgtflags


import warnings

with warnings.catch_warnings():
    warnings.filterwarnings("ignore", category=RuntimeWarning, module="importlib._bootstrap")
    # your code here

class NotCompatibleException(Exception):
    pass

#from jgtfxcon.common_samples import common_samples
from .jgtetl import svc_offset_dt_by_tf as etl_offset_dt_by_tf,offsetdt as etl_offsetdt
from .JGTCore import __version__ #,json2dict,jsonfile2prop,json2prop,jsonfile2dict,d2p,fixdtindf,offsetdt
#from .JGTConfig import getenv,setreal,setdemo,env

if platform.system() == 'Linux':
  #sys.path.append(os.path.abspath('./'))

  origin_work_dir = os.getcwd()
  here = os.path.abspath(os.path.dirname(__file__))
  os.chdir(here)
  try:
     from . import forexconnect
  except:
     from jgtfxcon import forexconnect 
  os.chdir(origin_work_dir)   

else:
  try:
    try:
      from . import forexconnect       
    except:
     from jgtfxcon import forexconnect 
  except:
    print("----------------------------------------------------------------")
    print("---Failed to load forexconnect --- Please Install forexconnect")
    print("--------- > pip install forexconnect (only an python =< 3.7)")
    print("--------")
    print("-----WINDOWS USER : ----")
    print("--If you are on an above Windows Python 3.7, it wont work.  ")
    print("--I made forexconnect to work on later than 3.7 only on Linux, ")
    print("-- sorry guys, migrate on Linux ;) or get involved migrating it ;) ")
    print("-----------------------------------------")
    raise NotCompatibleException("Forexconnect is not compatible with your current environment.")



# os.chdir(origin_work_dir)   
from .jgtfxcommon import _JGT_CONFIG_JSON_SECRET


from .JGTPDS import getPH as get_price, stayConnectedSetter as set_stay_connected, disconnect,connect as on,disconnect as off, status as connection_status,  getPH2file as get_price_to_file, getPHByRange as get_price_range, stayConnectedSetter as sc,getPH as ph,getPH_to_filestore as ph2fs
def stay():
    sc(True)
def up():
    sc(False)
def h(instrument,timeframe,quote_count=335,start=None,end=None,quiet=True):
        stay()
        #df= ph(instrument,timeframe,quote_count,start,end,False,quiet)
        fpath,df = ph2fs(instrument,timeframe,quote_count,start,end,False,quiet)
        return df

# from .JGTIDS import tocds as tocds
# #mk_fn,mk_fullpath,getSubscribed,getPH,getPHByRange,tryConnect
# from .JGTCDS import create as createCDS,createByRange,createFromDF,startSession,stopSession,getLast,getPresentBar,getPresentBarAsList,getLastCompletedBarAsList,createFromDF,createFromFile_and_clean_and_save_data as fromfile,createFromFile_and_clean_and_save_data as ff,createFromDF as fp

def help():
    print(".h(i,t,400)\t\tGet Prices (PDS): \n\t\t\tjgtfxcon.h(instrument,timeframe,quote_count=335,start=None,end=None,quiet=True)")
    print("\t\t\t\t\treturn DataFrame of PDS Type")
    print("\t\t\t\t\t(Will connect and stay connected)")
    print(".off()\t\t Disconnect (Bugged)\n\t\t\tjgtfxcon.off()")

# from jgtfxcon.JGTADS import ads_chart_pto,retrieve_n_chart as ads_retrieve_n_chart

# from .jgtcli import main as __main__
#from jgtfxcon.jgtconstants import *




# jgtfxc.py

import os
import platform

import json
#import datetime
from datetime import datetime,timezone
import pandas as pd

import warnings

with warnings.catch_warnings():
    warnings.filterwarnings("ignore", category=RuntimeWarning, module="importlib._bootstrap")
    # your code here


from .jgtfxcommon import get_connection_status,get_connection_status

from . import jgtflags

# origin_work_dir = os.getcwd()
# here = os.path.abspath(os.path.dirname(__file__))
# print("-----------------------------")
# print(here)
# print("-----------------------------")

# path_forexconnect = os.path.join(here, "forexconnect")

# if platform.system() == 'Windows':
#     path_forexconnect = os.path.join(here, 'lib', 'windows', 'mylib.dll')
# elif platform.system() == 'Linux':
#     path_forexconnect = os.path.join(here, 'lib', 'linux', 'mylib.so')
# else:
#     raise RuntimeError('Unsupported platform')


# os.chdir(path_forexconnect)

# from forexconnect import ForexConnect, fxcorepy
if platform.system() == 'Windows':
    from .forexconnect import ForexConnect, fxcorepy
else: 
    if platform.system() == 'Linux':
        try:
            # Try to import ForexConnect and fxcorepy from jgtpy.forexconnect
            from jgtfxcon.forexconnect import ForexConnect, fxcorepy
        except ModuleNotFoundError:
            # If that fails, try to import them directly
            try:
                from .forexconnect import ForexConnect, fxcorepy
            except ModuleNotFoundError:
                # If that also fails, print an error message
                print("Could not import ForexConnect or fxcorepy. Please ensure the modules are installed and available.")



# os.chdir(origin_work_dir)   

#@STCGoal Future use
#import forexconnect as fxcon
# fxcon.ForexConnect.create_order_request
# fxcon.ForexConnect.create_request
# fxcon.ForexConnect.get_table
# fxcon.ForexConnect.get_timeframe
# fxcon.ForexConnect.get_timeframe

# from . import common_samples as jgtfxcommon
from . import jgtfxcommon


fx=None
quotes_count=None
stayConnected=False
session=None
#session_status=None
#def get_session_status():
    #return get_connection_status()

def login_forexconnect(user_id, password, url, connection, quiet=False):
    global session,fx    
    jgtfxcommon.quiet=quiet
    if fx is not None:
        try:
            fx.logout()
        except:
            pass
    fx = ForexConnect()
    try:
        fx.login(user_id=user_id,password=password,url=url,connection=connection, pin="", session_id="", session_status_callback=jgtfxcommon.session_status_changed)
        #session_status= jgtfxcommon.get_connection_status()
    except Exception as e:
        jgtfxcommon.print_exception(e)
        print("------bahhhhhhhhhhhh----------")
    return fx
_config=None
#@STCIssue Matching our original connect
def connect(quiet=True,json_config_str=None):
    global fx,quotes_count,_config
    
    if fx is not None or get_connection_status()== "CONNECTED":
        if not quiet:
            print("Already connected")
        return
    
    if _config is None:
        _config=readconfig(json_config_str)

    str_user_id = _config['user_id']
    str_password = _config['password']
    str_url = _config['url']
    str_connection = _config['connection']
    quotes_count = _config['quotes_count']
    

    fx = login_forexconnect(str_user_id, str_password, str_url, str_connection,quiet=quiet)
    
    return fx


def logout_forexconnect(fx,quiet=False):
    try:
        if fx is not None:
            fx.logout()
        fx=None
        #@STCIssue on How we deal with status
        #session_status=get_connection_status()
        #session_status="DISCONNECTED"
        return True
    except Exception as e:
        jgtfxcommon.print_exception(e)
        fx=None
        return False

def disconnect(quiet=True):
    global fx
    if fx is None:
        print_quiet(quiet,"Not connected")
        return True
    return logout_forexconnect(fx,quiet)


def status(quiet=True):
    return get_session_status()
    # if session is None:
    #     print_quiet(quiet,"UNKNOWN STATUS...")
    #     return False
    # else :
    #     print("---------AO2GSessionStatus-----------")
    #     print(fxcorepy.AO2GSessionStatus)
    #     print(session.AO2GSessionStatus)
    #     print("--------------------")
    #     if session.getStatus() == fxcorepy.AO2GSessionStatus.CONNECTED:
    #         print_quiet(quiet,"CONNECTED...")
    #         return True
    #     if session.getStatus() == fxcorepy.AO2GSessionStatus.DISCONECTED:
    #         print_quiet(quiet,"DISCONNECTED...")
    #         return False
    # print_quiet(quiet,"UNKNOWN STATUS...")
    # return False
        
def status1(quiet=True):
    global fx
    if fx is None:
        print_quiet(quiet,"STATUS : Not Connected")
        return False
    print_quiet(quiet,"STATUS : Connected")
    return True

def print_quiet(quiet,content):
    if not quiet:
        print(content)

_config=None

def readconfig(json_config_str=None):
    global _config
    # # Try reading config file from current directory
    # config_file = 'config.json'
    # if not os.path.isfile(config_file):
    #     # If config file not found, check home directory
    #     home_dir = os.path.expanduser("~")
    #     config_file = os.path.join(home_dir, 'config.json')

    # # Read config file
    # with open(config_file, 'r') as file:
    if _config is None:
        _config = jgtfxcommon.readconfig(json_config_str)
    return _config


def get_price_history(instrument, timeframe, datefrom=None, dateto=None,quotes_count_spec=None,quiet=True):
    global quotes_count,fx

    if quotes_count_spec is None:
        quotes_count_spec=quotes_count

    connect(quiet=quiet)
    # if home_dir/.jgt/iprops make it and run a save of this instrument properties
    iprop=get_instrument_properties(instrument,quiet)
    
    try:
        print_quiet(quiet,"Requesting a price history...")
  

        if datefrom is not None:
            date_from_parsed = parse_date(datefrom)
        else:
            date_from_parsed=None
        
        if dateto is None:
            date_to_parsed = datetime.now(timezone.utc)
        else:
            date_to_parsed = parse_date(dateto)
        
        if not quiet:
            print("Date from : " + str(date_from_parsed))
            print("Date to : " + str(date_to_parsed))


        history = fx.get_history(instrument, timeframe, date_from_parsed, date_to_parsed, quotes_count_spec)

        current_unit, _ = ForexConnect.parse_timeframe(timeframe)

        if current_unit == fxcorepy.O2GTimeFrameUnit.TICK:
            data = pd.DataFrame(history, columns=['Date', 'Bid', 'Ask'])
        else:
            data = pd.DataFrame(history, columns=['Date','BidOpen','BidHigh','BidLow','BidClose','AskOpen','AskHigh','AskLow','AskClose','Volume'])

        return data

    finally:
        if not stayConnected:
            disconnect()
        else:
            print("---we stay connected---")
        #logout_forexconnect(fx)



def get_price_history_printed(instrument, timeframe, datefrom=None, dateto=None):
    data = get_price_history(instrument, timeframe, datefrom, dateto)

    if data is not None:
        if 'Ask' in data.columns:
            print("Date, Bid, Ask")
        else:
            print("Date, BidOpen, BidHigh, BidLow, BidClose, Volume")

        for index, row in data.iterrows():
            values = row.values.tolist()
            print(",".join(str(value) for value in values))


# Example usage
#get_price_history_printed(instrument='EUR/USD', timeframe='m1')

#fx
def getAccount():
    # account = fx.getAccount()
    # print(account)
    # return account
    print("Not implemented yet")

def getSubscribedSymbols():
    if fx is None:
        connect()
    # symbols = fx.getSubscribedSymbols()
    # print(symbols)
    #return symbols
    print("Not implemented yet")


def get_pipsize(s_instrument):
    if fx is None:
        connect()
    table_manager = fx.table_manager
    offers_table = table_manager.get_table(ForexConnect.OFFERS)
    for offer_row in offers_table:
        if offer_row.instrument == s_instrument:
            return offer_row.PointSize



def parse_date(date_str):
    if date_str is None:
        for fmt in ('%d.%m.%Y %H:%M:%S', '%d.%m.%Y %H:%M','%d.%m.%Y','%Y%m%d%H%M','%y%m%d%H%M','%Y-%m-%d %H:%M'):
            try:
                return datetime.strptime(date_str, fmt)
            except ValueError:
                pass
        raise ValueError('no valid date format found')
    

def get_instrument_properties(instrument, quiet=False):
    # Define the path to the directory
    home_dir = os.path.expanduser("~")
    dir_path = os.path.join(home_dir, '.jgt', 'iprops')
    instrument_properties = {}
    instrument_filename = instrument.replace('/', '-')
    
    # Check if the directory exists
    if not os.path.exists(dir_path):
        # If not, create it
        os.makedirs(dir_path)
    
    iprop_dir_path = os.path.join(dir_path, f'{instrument_filename}.json')
    # Check if the file exists
    if not os.path.exists(iprop_dir_path):
        # If not, create the directory if it doesn't exist
        if not os.path.exists(dir_path):
            os.makedirs(dir_path)

        # Define the instrument properties
        # Replace with your actual instrument properties
        pipsize = get_pipsize(instrument)
        instrument_properties = {
            "pipsize": pipsize
            # Add more properties as needed
        }

        # Replace forward slash with hyphen in the instrument name

        # Save the instrument properties to the file
        with open(iprop_dir_path, 'w') as f:
            json.dump(instrument_properties, f)

        if not quiet:
            print(f"Instrument properties for {instrument} saved.")
    else:
        # Read the instrument properties from the file
        with open(iprop_dir_path, 'r') as f:
            instrument_properties = json.load(f)

        if not quiet:
            print(f"Instrument properties for {instrument} read.")
    return instrument_properties
    



from . import env,jsonfile2prop,d2p,createByRange
import pandas as pd
from functools import reduce
from . import jgtetl

ctx=d2p(env)

trainsetfnjson=env['trainfnjson'] 
trainsetfncsv=env['trainfncsv'] 

dtlast = ""
if hasattr(ctx, 'dtlast'):
  dtlast=ctx.dtlast
  print(dtlast)
else:
  dst=jsonfile2prop(trainsetfnjson)
  l=0
  for key, value in dst.items():
    l=key
  lastitem=d2p(dst[l])
  # print(lastitem)
  # print(lastitem.state)
  # print(lastitem.plx)
  # print(lastitem.bs)
  dtlast= lastitem.dt
  print('export lastdt="' + lastitem.dt+ '"')

print("The variable, dtlast is of type:", type(dtlast))
print("The variable, ctx.firstst is of type:", type(ctx.firstst))
dtfirst=ctx.firstst
dtfirst_with_offset=jgtetl.svc_offset_dt_by_tf(dtfirst,ctx.timeframe)

# jgtetl.offsetdt
#pto_nbhourbytfofoffset2212
# ctx['lastdt']=lastitem.dt
print("CReating CDS with Range: " + dtfirst_with_offset + " > " + dtlast)
df=createByRange(ctx.instrument,ctx.timeframe,dtfirst_with_offset,dtlast)
fnids=ctx.tsfnbase+".ids.csv"
df.to_csv(fnids)

#  SEE PART 02

# fnbase=ctx.tsfnbase
# # fn=fnbase + ".trainset.csv"
# fn=trainsetfncsv

# fnfdbb=fnbase + ".fdb."+ctx.bs+".csv"
# fnplx=fnbase + ".plx.csv"
# fnrrstotc=fnbase + ".rrstotc.csv"
# #df=pd.DataFrame()
# df=pd.read_csv(fn)
# df.index.rename('no',inplace=True)
# dfsel=pd.DataFrame()
# fdbTag='FDBB'
# if ctx.bs=='Sell':
#   fdbTag='FDBS'
# dfsel=df[df['sig']== fdbTag]
# dfsel.to_csv(fnfdbb)
# dfplx=dfsel[['bp','state','plx','plxc','tpl','tplc','to','tc','bs','dt','plxdt','i','tdt','oid','tid','r','ro','rs','rsx','sig','tf','tm']]
# totplx=dfplx['plx'].sum()
# print(totplx)
# dfrrstotc=dfsel[['bp','state','plx','tpl','r','to','rs','tc','ro','plxc','tplc','bs','dt','plxdt','i','tdt','oid','tid','rsx','sig','tf','tm']]
# #dfsel.index.rename('bp',inplace=True)
# dfplx.to_csv(fnplx)
# dfrrstotc.to_csv(fnrrstotc)



import JGTPDS as pds




def place_order(instrument, quantity, order_type, price=None):
  """
  Places an order.

  Args:
  instrument: The instrument to trade.
  quantity: The quantity to trade.
  order_type: The type of the order (e.g., 'market', 'limit').
  price: The price at which to execute the order (only for limit orders).

  Returns:
  The ID of the placed order.
  """
  # Implement the logic to place the order here
  # Return the ID of the placed order
  pass

def cancel_order(order_id):
  """
  Cancels an order.

  Args:
  order_id: The ID of the order to cancel.

  Returns:
  A boolean indicating whether the cancellation was successful.
  """
  # Implement the logic to cancel the order here
  # Return whether the cancellation was successful
  pass

def get_order_status(order_id):
  """
  Gets the status of an order.

  Args:
  order_id: The ID of the order.

  Returns:
  The status of the order.
  """
  # Implement the logic to get the order status here
  # Return the status of the order
  pass

def get_open_orders():
  """
  Gets all open orders.

  Returns:
  A list of all open orders.
  """
  # Implement the logic to get all open orders here
  # Return the list of open orders
  pass

def get_trade_history():
  """
  Gets the trade history.

  Returns:
  A list of all past trades.
  """
  # Implement the logic to get the trade history here
  # Return the list of past trades
  pass


def get_price_plus_minus_ticks(instrument, ticks_multiplier, context_price, direction_side):
  """
  Gets the price value plus or minus a defined number of ticks.

  Args:
  instrument: The instrument to trade.
  ticks_multiplier: The number of ticks to add or subtract.
  context_price: The current price of the instrument.
  direction_side: The direction side to use ('S' for minus, 'B' for plus).

  Returns:
  The price value plus or minus the defined number of ticks.
  """
  instrument_properties = pds.get_instrument_properties(instrument)
  tick_size = instrument_properties.pipsize * ticks_multiplier
  if direction_side == 'S':
    price_minus_ticks = context_price - (ticks_multiplier * tick_size)
    return price_minus_ticks
  elif direction_side == 'B':
    price_plus_ticks = context_price + (ticks_multiplier * tick_size)
    return price_plus_ticks
  else:
    raise ValueError("Invalid direction side. Must be 'S' or 'B'.")


